/*****************************************************************************/
/*                      WiNRADiO G35DDC EXCALIBUR ULTRA                      */
/*                                SDK DEMO                                   */
/*                             Main GUI window                               */
/*                        (C) Copyright RADIXON 2014                         */
/*****************************************************************************/

#include "u_mainwindow.h"
#include "u_defs.h"
#include "u_global.h"


enum
{
    ID_POWER=wxID_HIGHEST,
    ID_ATTENUATOR,
    ID_PREAMP,
    ID_PRESEL_CUT_OFF_LOW,
    ID_PRESEL_CUT_OFF_HIGH,
    ID_IFRBW,
    ID_IF_SPECTRUM,
    ID_POWER_STATE_TIMER,
    ID_CONN_TIMER,
    ID_INIT_POWER_ON_TIMER,    
    ID_DDC1_SPECTRUM,
    ID_DDC1BW,
    ID_DDC1RBW,
    ID_RX_BASE=wxID_HIGHEST+100    
};



BEGIN_EVENT_TABLE(TMainWindow, wxFrame)
    EVT_TOGGLEBUTTON(ID_POWER,TMainWindow::OnButtonClick) 
    EVT_TOGGLEBUTTON(ID_PREAMP,TMainWindow::OnButtonClick) 
    EVT_COMBOBOX(ID_IFRBW,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_ATTENUATOR,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_PRESEL_CUT_OFF_LOW,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_PRESEL_CUT_OFF_HIGH,TMainWindow::OnComboBoxChanged)
    EVT_SPECTRUM_TUNE(ID_IF_SPECTRUM,TMainWindow::OnSpectrumTune)
    EVT_TIMER(ID_POWER_STATE_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_CONN_TIMER,TMainWindow::OnTimer)
    EVT_TIMER(ID_INIT_POWER_ON_TIMER,TMainWindow::OnTimer)
    EVT_COMBOBOX(ID_DDC1BW,TMainWindow::OnComboBoxChanged)
    EVT_COMBOBOX(ID_DDC1RBW,TMainWindow::OnComboBoxChanged)
    EVT_SPECTRUM_TUNE(ID_DDC1_SPECTRUM,TMainWindow::OnSpectrumTune)
END_EVENT_TABLE()


TMainWindow::TMainWindow(TReceiver *Receiver):m_Receiver(Receiver),m_PowerStateTimer(this,ID_POWER_STATE_TIMER),
m_ConnTimer(this,ID_CONN_TIMER),m_InitPowerOnTimer(this,ID_INIT_POWER_ON_TIMER),wxFrame(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition,wxSize(900,800))
{
 uint32_t i;
 wxString Title,s;    
    
    m_ActiveRXPanel=NULL;

    m_DeviceInfo=Receiver->GetDeviceInfo();

    SetFont(*Global.m_Font);
    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));

    Title.sprintf(APP_TITLE wxT(" - %s"),wxString(m_DeviceInfo.SerialNumber,wxConvLocal).c_str());

    SetTitle(Title);

    m_MainSizer=new wxBoxSizer(wxVERTICAL);

    m_IFSizerTop=new wxBoxSizer(wxHORIZONTAL);
    m_IFSizerBottom=new wxBoxSizer(wxHORIZONTAL);

    m_DDC1Sizer=new wxBoxSizer(wxVERTICAL);
    m_DDC1SizerTop=new wxBoxSizer(wxHORIZONTAL);
    m_DDC1Sizer->Add(m_DDC1SizerTop,0,wxEXPAND|wxALIGN_TOP|wxBOTTOM,4);
        
    m_RXSizer=new wxBoxSizer(wxHORIZONTAL);
    m_MainSizer->Add(m_RXSizer,0,wxEXPAND|wxALIGN_LEFT|wxALIGN_TOP|wxALL,4);

    m_BRX=new wxToggleButton*[m_DeviceInfo.ChannelCount];

    m_RXPanels=new TRXPanel*[m_DeviceInfo.ChannelCount];

    for(i=0;i<m_DeviceInfo.ChannelCount;i++)    
    {
        s.sprintf(wxT("RX %u"),i+1);
        m_BRX[i]=new wxToggleButton(this,ID_RX_BASE+i,s,wxDefaultPosition,wxSize(47,23));
        m_RXSizer->Add(m_BRX[i],0,wxRIGHT,4);

        m_RXPanels[i]=new TRXPanel(this,m_Receiver,i);
        m_MainSizer->Add(m_RXPanels[i],1,wxEXPAND|wxLEFT|wxRIGHT,4);

        if(i!=0)
        {
            m_RXPanels[i]->Hide();
        }
    }
    
    Connect(ID_RX_BASE,ID_RX_BASE+m_DeviceInfo.ChannelCount-1,wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,wxCommandEventHandler(TMainWindow::OnRXButtonClick));

    m_MainSizer->Add(m_DDC1Sizer,0,wxALIGN_TOP|wxEXPAND|wxRIGHT|wxLEFT|wxTOP,4);

    m_LDDC1BW=new wxStaticText(this,wxID_ANY,wxT("DDC1 BW"));
    m_DDC1SizerTop->Add(m_LDDC1BW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC1SizerTop->AddSpacer(4);

    m_CBDDC1BW=new wxComboBox(this,ID_DDC1BW,wxEmptyString,wxDefaultPosition,wxSize(75,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_DDC1SizerTop->Add(m_CBDDC1BW,0,wxALIGN_LEFT);

    m_DDC1SizerTop->AddSpacer(8);

    m_LDDC1RBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_DDC1SizerTop->Add(m_LDDC1RBW,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_DDC1SizerTop->AddSpacer(4);

    m_CBDDC1RBW=new wxComboBox(this,ID_DDC1RBW,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_DDC1SizerTop->Add(m_CBDDC1RBW,0,wxALIGN_LEFT);

    m_DDC1Spectrum=new TSpectrum(this,ID_DDC1_SPECTRUM,wxDefaultPosition,wxSize(100,160));
    m_DDC1Spectrum->SetMinLevel(-160);
    m_DDC1Spectrum->SetBand(SpectrumBand_BothSides,true);
    m_DDC1Spectrum->SetText(wxT("DDC1 spectrum"));
    m_DDC1Sizer->Add(m_DDC1Spectrum,1,wxEXPAND);

    m_MainSizer->Add(m_IFSizerTop,0,wxALIGN_TOP|wxALL|wxEXPAND,4);    

    m_LRBW=new wxStaticText(this,wxID_ANY,wxT("RBW"));
    m_IFSizerTop->Add(m_LRBW,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSizerTop->AddSpacer(4);

    m_CBIFRBW=new wxComboBox(this,ID_IFRBW,wxEmptyString,wxDefaultPosition,wxSize(75,23),0,0,wxCB_DROPDOWN|wxCB_READONLY|wxTE_CENTRE);
    m_IFSizerTop->Add(m_CBIFRBW,0,wxALIGN_LEFT|wxCENTRE);

    m_IFSpectrum=new TSpectrum(this,ID_IF_SPECTRUM,wxDefaultPosition,wxSize(100,120));
    m_IFSpectrum->SetText(wxT("IF spectrum"));
    m_IFSpectrum->SetMinLevel(-140);
    m_MainSizer->Add(m_IFSpectrum,0,wxEXPAND|wxLEFT|wxRIGHT,4);

    m_MainSizer->Add(m_IFSizerBottom,0,wxALIGN_BOTTOM|wxEXPAND|wxALL,4);       

    m_IFSizerBottom->AddSpacer(4);

    m_LAttenuator=new wxStaticText(this,wxID_ANY,wxT("Attenuator"));
    m_IFSizerBottom->Add(m_LAttenuator,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_IFSizerBottom->AddSpacer(4);

    m_CBAttenuator=new wxComboBox(this,ID_ATTENUATOR,wxEmptyString,wxDefaultPosition,wxSize(60,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_IFSizerBottom->Add(m_CBAttenuator,0,wxALIGN_LEFT);

    m_IFSizerBottom->AddSpacer(4);

    m_BPreamp=new wxToggleButton(this,ID_PREAMP,wxT("Preamplifier"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerBottom->Add(m_BPreamp,0,wxALIGN_LEFT);

    m_IFSizerBottom->AddSpacer(8);

    m_LPresel=new wxStaticText(this,wxID_ANY,wxT("Preselectors"));
    m_IFSizerBottom->Add(m_LPresel,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_IFSizerBottom->AddSpacer(4);

    m_CBPresel_CutOffLow=new wxComboBox(this,ID_PRESEL_CUT_OFF_LOW,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_IFSizerBottom->Add(m_CBPresel_CutOffLow,0,wxALIGN_LEFT);

    m_IFSizerBottom->AddSpacer(4);

    m_LPresel2=new wxStaticText(this,wxID_ANY,wxT("-"));
    m_IFSizerBottom->Add(m_LPresel2,0,wxALIGN_LEFT|wxALIGN_CENTRE);

    m_IFSizerBottom->AddSpacer(4);

    m_CBPresel_CutOffHigh=new wxComboBox(this,ID_PRESEL_CUT_OFF_HIGH,wxEmptyString,wxDefaultPosition,wxSize(70,23),0,0,wxCB_DROPDOWN|wxCB_READONLY);
    m_IFSizerBottom->Add(m_CBPresel_CutOffHigh,0,wxALIGN_LEFT);

    m_IFSizerBottom->Add(0,1,1,wxALIGN_LEFT);

    m_BPower=new wxToggleButton(this,ID_POWER,wxT("Power"),wxDefaultPosition,wxSize(70,23));
    m_IFSizerBottom->Add(m_BPower,0,wxRIGHT);
        

    SetSizer(m_MainSizer);


    Initialize();

    SetClientSize(748,750);
    SetMinSize(wxSize(GetSize().GetWidth(),GetSize().GetHeight()));

    //set receiver callbacks to this object
    //it is done after Initialize() to prevent from many calls of TReceiver methods
    //during initialization of controls which could invoke many receiver/channel events
    m_Receiver->SetCallback(this);

    m_ConnTimer.Start(500);

    //power on with delay
    m_InitPowerOnTimer.Start(200,true);        

    Layout();
    Centre();
}

TMainWindow::~TMainWindow(void)
{
    delete m_Receiver;

    delete[] m_RXPanels;
    delete[] m_BRX;
}

//ReceiverCallback_Event is called when a receiver event occurs (e.g. power state changed, DDC1 frequency changed,...)
void TMainWindow::ReceiverCallback_Event(TReceiverEvent Event)
{
 uint32_t i;

    switch(Event)
    {
        case ReceiverEvent_Power:
            UpdateState();
            break;

        case ReceiverEvent_Attenuator:
            UpdateAttenuatorState();
            break;

        case ReceiverEvent_Preamplifier:
            UpdatePreamplifierState();
            break;

        case ReceiverEvent_Preselectors:
            UpdatePreselectorState();
            break;

        case ReceiverEvent_IFRBWIndex:
            UpdateIFRBWIndex();
            break;

        case ReceiverEvent_ActiveChannel:
            UpdateActiveChannel();
            break;

        case ReceiverEvent_DDC1:
            UpdateDDC1();
            break;

        case ReceiverEvent_DDC1_Frequency:
            UpdateDDC1Frequency();
            break;

        case ReceiverEvent_DDC1_RBWList:
            UpdateDDC1RBWList();
            break;

        case ReceiverEvent_DDC1_RBWIndex:
            UpdateDDC1RBW();
            break;        

        case ReceiverEvent_DDC2:
            UpdateDDC2();

            for(i=0;i<m_DeviceInfo.ChannelCount;i++)
            {
                m_RXPanels[i]->UpdateDDC2();
            }
            break;

        case ReceiverEvent_DDC2_RBWList:
            for(i=0;i<m_DeviceInfo.ChannelCount;i++)
            {
                m_RXPanels[i]->UpdateDDC2RBWList();
            }
            break;
    }
}

void TMainWindow::ReceiverCallback_ChannelEvent(uint32_t Channel,TChannelEvent Event)
{   
    switch(Event)
    {
        case ChannelEvent_DDC2_Frequency:
            UpdateDDC2Frequency(Channel);
            break;
    }

    //forwatd event to RXPanel
    m_RXPanels[Channel]->ReceiverCallback_ChannelEvent(Event);
}


void TMainWindow::Initialize(void)
{
 uint32_t i,Count;
 G3XDDC_DDC_INFO DDCInfo;
 int RBW,CutOff;
 wxString s;
 double RBWs[RBW_COUNT];


    m_Receiver->GetIFRBWs(RBWs);

    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=(int)(RBWs[i]+0.5);

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        s=wxT(" ")+FormatFrequency(RBW);

        m_CBIFRBW->Append(s);
    }

    m_CBAttenuator->Append(wxT(" Off"));
    m_CBAttenuator->Append(wxT(" 3 dB"));
    m_CBAttenuator->Append(wxT(" 6 dB"));
    m_CBAttenuator->Append(wxT(" 9 dB")); 
    m_CBAttenuator->Append(wxT(" 12 dB")); 
    m_CBAttenuator->Append(wxT(" 15 dB")); 
    m_CBAttenuator->Append(wxT(" 18 dB")); 
    m_CBAttenuator->Append(wxT(" 21 dB")); 

    for(i=0;i<PRESEL_COUNT;i++)
    {
        CutOff=m_Receiver->GetPreselCutOffLow(i);
        s=wxT(" ")+FormatFrequency(CutOff);
        m_CBPresel_CutOffLow->Append(s);


        CutOff=m_Receiver->GetPreselCutOffHigh(i);
        s=wxT(" ")+FormatFrequency(CutOff);
        m_CBPresel_CutOffHigh->Append(s);
    }

    m_IFSpectrum->SetRange(50000000);
    m_IFSpectrum->SetCenterFrequency(25000000);
    m_IFSpectrum->SetBand(SpectrumBand_BothSides,false);
        
    UpdateAttenuatorState();
    UpdatePreamplifierState();
    UpdatePreselectorState();
    UpdateIFRBWIndex();
    

    Count=m_Receiver->GetDDC1Count();

    for(i=0;i<Count;i++)
    {
        m_Receiver->GetDDCInfo(i,&DDCInfo);

        m_CBDDC1BW->Append(wxT(" ")+FormatFrequency(DDCInfo.Bandwidth));        
    }

    for(i=0;i<m_DeviceInfo.ChannelCount;i++)
    {
        m_RXPanels[i]->Initialize();
    }    

    UpdateDDC1();
    UpdateDDC1Frequency();
    UpdateDDC1RBWList();
    UpdateDDC2();


    UpdateState();

    UpdateActiveChannel();
}

void TMainWindow::OnRXButtonClick(wxCommandEvent &Event)
{
 uint32_t Channel;

    Channel=Event.GetId()-ID_RX_BASE;

    if(Channel==m_Receiver->GetActiveChannel())
    {
        m_BRX[Channel]->SetValue(true);
    }
    else
    {
        m_Receiver->SetActiveChannel(Channel);            
    }
}

void TMainWindow::OnButtonClick(wxCommandEvent &Event)
{
 wxBusyCursor *BusyCursor;

    switch(Event.GetId())
    {
        case ID_POWER:
            BusyCursor=new wxBusyCursor;

            m_BPower->Update();

            m_Receiver->SetPower(m_BPower->GetValue());

            delete BusyCursor;
            break;

        case ID_PREAMP:
            m_Receiver->SetPreamplifier(m_BPreamp->GetValue());
            break;        
    }
}

void TMainWindow::OnComboBoxChanged(wxCommandEvent &Event)
{
 uint32_t i; 

    switch(Event.GetId())
    {
        case ID_DDC1BW:
            if(m_CBDDC1BW->GetSelection()>=0)
            {
                m_Receiver->SetDDC1(m_CBDDC1BW->GetSelection());
            }
            break;

        case ID_DDC1RBW:
            if(m_CBDDC1RBW->GetSelection()>=0)
            {
                m_Receiver->SetDDC1RBWIndex(m_CBDDC1RBW->GetSelection());
            }
            break;

        case ID_ATTENUATOR:
            m_Receiver->SetAttenuator(m_CBAttenuator->GetSelection());
            break;

        case ID_IFRBW:
            m_Receiver->SetIFRBWIndex(m_CBIFRBW->GetSelection());
            break;

        case ID_PRESEL_CUT_OFF_LOW:
            i=m_CBPresel_CutOffHigh->GetSelection();

            while(m_Receiver->GetPreselCutOffLow(m_CBPresel_CutOffLow->GetSelection())>m_Receiver->GetPreselCutOffHigh(i))
            {
                i++;                
            }

            m_Receiver->SetPreselectors(m_CBPresel_CutOffLow->GetSelection(),i);
            break;

        case ID_PRESEL_CUT_OFF_HIGH:
            i=m_CBPresel_CutOffLow->GetSelection();

            while(m_Receiver->GetPreselCutOffLow(i)>m_Receiver->GetPreselCutOffHigh(m_CBPresel_CutOffHigh->GetSelection()))
            {
                i--;
            }
            
            m_Receiver->SetPreselectors(i,m_CBPresel_CutOffHigh->GetSelection());
            break;
    }
}

void TMainWindow::UpdateState(void)
{
 bool Power;
 bool Enabled;
 bool Connected;
 uint32_t i;

    Power=m_Receiver->GetPower();
    Connected=m_Receiver->IsConnected();

    if(Power && Connected)
    {
        m_PowerStateTimer.Start(500);
    }
    else
    {
        m_PowerStateTimer.Stop();
    }

    Enabled=Power && Connected;

    m_BPower->SetValue(Power);
    m_IFSpectrum->Enable(Enabled);
    m_IFSpectrum->Clear();

    m_LRBW->Enable(Enabled);
    m_CBIFRBW->Enable(Enabled);

    m_LDDC1BW->Enable(Enabled);
    m_CBDDC1BW->Enable(Enabled);
    m_LDDC1RBW->Enable(Enabled);
    m_CBDDC1RBW->Enable(Enabled);

    m_DDC1Spectrum->Enable(Enabled);    

    UpdateAttenuatorState();
    UpdatePreamplifierState();
    UpdatePreselectorState();

    for(i=0;i<m_DeviceInfo.ChannelCount;i++)
    {
        m_RXPanels[i]->UpdateState();       
    }

    if(!Enabled)
    {
        m_DDC1Spectrum->Clear();
        m_IFSpectrum->Clear();
    }
}



void TMainWindow::UpdateAttenuatorState(void)
{
 bool Enabled;

    m_CBAttenuator->SetSelection(m_Receiver->GetAttenuator());

    Enabled=m_Receiver->GetPower();

    m_CBAttenuator->Enable(Enabled);
    m_LAttenuator->Enable(Enabled);
}

void TMainWindow::UpdatePreamplifierState(void)
{
    m_BPreamp->SetValue(m_Receiver->GetPreamplifier());

    m_BPreamp->Enable(m_Receiver->GetPower());
}

void TMainWindow::UpdatePreselectorState(void)
{
 uint32_t LowIndex,HighIndex;
 bool Enabled;

    m_Receiver->GetPreselectors(&LowIndex,&HighIndex);

    m_CBPresel_CutOffLow->SetSelection(LowIndex);
    m_CBPresel_CutOffHigh->SetSelection(HighIndex);

    Enabled=m_Receiver->GetPower();

    m_LPresel->Enable(Enabled);
    m_CBPresel_CutOffLow->Enable(Enabled);
    m_LPresel2->Enable(Enabled);
    m_CBPresel_CutOffHigh->Enable(Enabled);
}

void TMainWindow::ReceiverCallback_IFSpectrum(float *Data,uint32_t Count,double Range,double Offset)
{
    m_IFSpectrum->SetData(Data,Count,Range,Offset);    
}

void TMainWindow::ReceiverCallback_DDC1Spectrum(float *Data,uint32_t Count,double Range,double Offset)
{
    m_DDC1Spectrum->SetData(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallback_DDC2Spectrum(uint32_t Channel,float *Data,uint32_t Count,double Range,double Offset)
{
    m_RXPanels[Channel]->ReceiverCallback_DDC2Spectrum(Data,Count,Range,Offset);
}

void TMainWindow::ReceiverCallback_AudioSpectrum(uint32_t Channel,float *Data,uint32_t Count,uint32_t Range)
{
    m_RXPanels[Channel]->ReceiverCallback_AudioSpectrum(Data,Count,Range);
}

void TMainWindow::UpdateIFRBWIndex(void)
{
    m_CBIFRBW->SetSelection(m_Receiver->GetIFRBWIndex());
}

void TMainWindow::ReceiverCallback_Audio(uint32_t Channel,const float *Samples,uint32_t Count)
{
    m_RXPanels[Channel]->ReceiverCallback_Audio(Samples,Count);
}

void TMainWindow::UpdateActiveChannel(void)
{
 uint32_t Channel,i;
 TRXPanel *Panel;

    Channel=m_Receiver->GetActiveChannel();

    if(m_ActiveRXPanel!=m_RXPanels[Channel])
    {
        for(i=0;i<m_DeviceInfo.ChannelCount;i++)
        {
            m_BRX[i]->SetValue(i==Channel);
        }

        Panel=m_ActiveRXPanel;

        m_ActiveRXPanel=m_RXPanels[Channel];
        m_ActiveRXPanel->Show(true);

        if(Panel)
        {
            Panel->Show(false);
        }

        UpdateDDC2Frequency(Channel);

        Layout();
    }
}

void TMainWindow::UpdateDDC1Frequency(void)
{
    m_IFSpectrum->SetBandFrequency(m_Receiver->GetDDC1Frequency()); 
    m_DDC1Spectrum->SetCenterFrequency(m_Receiver->GetDDC1Frequency());
}

void TMainWindow::UpdateDDC1(void)
{    
    m_IFSpectrum->SetBandWidth(m_Receiver->GetDDC1Info().Bandwidth);

    m_CBDDC1BW->SetSelection(m_Receiver->GetDDC1());

    m_DDC1Spectrum->SetRange(m_Receiver->GetDDC1Info().Bandwidth);
}

void TMainWindow::OnSpectrumTune(TSpectrumTuneEvent &Event)
{
    switch(Event.GetId())
    {
        case ID_DDC1_SPECTRUM:
            m_Receiver->SetDDC2Frequency(m_Receiver->GetActiveChannel(),(int32_t)Event.GetFrequency());
            break;

        case ID_IF_SPECTRUM:
            m_Receiver->SetDDC1Frequency((int32_t)Event.GetFrequency());
            break;
    }
}

void TMainWindow::OnTimer(wxTimerEvent& Event)
{
 wxBusyCursor *BusyCursor;

    switch(Event.GetId())
    {
        case ID_POWER_STATE_TIMER:            
            if(!m_Receiver->GetPowerDirect())
            {
                wxMessageBox(wxT("WARNING: The receiver's temperature is too high.\n")
                             wxT("The receiver has been turned off to prevent damage.\n\n")
                             wxT("Please wait a moment and then turn the receiver on again.\n")
                             wxT("If the problem persists, please ensure that the receiver's fan\n")
                             wxT("and free air flow around the receiver are not obstructed."),wxT("Warning"),wxICON_WARNING|wxOK);
            }
            break;

        case ID_CONN_TIMER:
            if(!m_Receiver->IsConnected())
            {
                m_ConnTimer.Stop();

                UpdateState();
                wxMessageBox(APP_TITLE wxT(" will be closed.\nReceiver is no longer connected."),wxT("Warning"),wxICON_WARNING|wxOK);
                Close();
            }
            break;

        case ID_INIT_POWER_ON_TIMER:
            BusyCursor=new wxBusyCursor;

            m_BPower->SetValue(true);
            m_BPower->Update();

            m_Receiver->SetPower(true);                        

            delete BusyCursor;
            break;
    }
}


void TMainWindow::UpdateDDC1RBWList(void)
{
 int i;
 int RBW;
 double RBWs[RBW_COUNT];

    m_CBDDC1RBW->Clear();

    m_Receiver->GetDDC1RBWs(RBWs);
    
    for(i=0;i<RBW_COUNT;i++)
    {
        RBW=RBWs[i]+0.5;

        if(RBW>1000)
        {
            RBW+=50;
            RBW=(RBW/100)*100;
        }

        m_CBDDC1RBW->Append(wxT(" ")+FormatFrequency(RBW));
    }

    m_CBDDC1RBW->SetSelection(m_Receiver->GetDDC1RBWIndex());
}

void TMainWindow::UpdateDDC1RBW(void)
{
    m_CBDDC1RBW->SetSelection(m_Receiver->GetDDC1RBWIndex());
}

void TMainWindow::UpdateDDC2(void)
{
    m_DDC1Spectrum->SetBandWidth(m_Receiver->GetDDC2Info().Bandwidth);
}

void TMainWindow::UpdateDDC2Frequency(uint32_t Channel)
{
    if(m_Receiver->GetActiveChannel()==Channel)
    {
        m_DDC1Spectrum->SetBandFrequency(m_Receiver->GetDDC2Frequency(Channel));
    }
}
